@@ -86,7 +86,7 @@ static Tk_OptionSpec ItemOptionSpecs[] = {
  */
 static TreeItem *NewItem(void)
 {
-    TreeItem *item = (TreeItem*)ckalloc(sizeof(*item));
+    TreeItem *item = ckalloc(sizeof(*item));
 
     item->entryPtr = 0;
     item->parent = item->children = item->next = item->prev = NULL;
@@ -118,7 +118,7 @@ static void FreeItem(TreeItem *item)
     if (item->tagset)	{ Ttk_FreeTagSet(item->tagset); }
     if (item->imagespec) { TtkFreeImageSpec(item->imagespec); }
 
-    ckfree((ClientData)item);
+    ckfree(item);
 }
 
 static void FreeItemCB(void *clientData) { FreeItem(clientData); }
@@ -340,8 +340,8 @@ static int GetEnumSetFromObj(
 
     for (i = 0; i < objc; ++i) {
 	int index;
-	if (TCL_OK != Tcl_GetIndexFromObj(
-		    interp, objv[i], table, "value", TCL_EXACT, &index))
+	if (TCL_OK != Tcl_GetIndexFromObjStruct(interp, objv[i], table,
+		sizeof(char *), "value", TCL_EXACT, &index))
 	{
 	    return TCL_ERROR;
 	}
@@ -534,21 +534,18 @@ static TreeColumn *GetColumn(
      */
     if (Tcl_GetIntFromObj(NULL, columnIDObj, &columnIndex) == TCL_OK) {
 	if (columnIndex < 0 || columnIndex >= tv->tree.nColumns) {
-	    Tcl_ResetResult(interp);
-	    Tcl_AppendResult(interp,
-		    "Column index ",
-		    Tcl_GetString(columnIDObj),
-		    " out of bounds",
-		    NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "Column index %s out of bounds",
+		    Tcl_GetString(columnIDObj)));
+	    Tcl_SetErrorCode(interp, "TTK", "TREE", "COLBOUND", NULL);
 	    return NULL;
 	}
 
 	return tv->tree.columns + columnIndex;
     }
-    Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp,
-	"Invalid column index ", Tcl_GetString(columnIDObj),
-	NULL);
+    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+	"Invalid column index %s", Tcl_GetString(columnIDObj)));
+    Tcl_SetErrorCode(interp, "TTK", "TREE", "COLUMN", NULL);
     return NULL;
 }
 
@@ -566,10 +563,9 @@ static TreeColumn *FindColumn(
 	    return tv->tree.displayColumns[colno];
 	}
 	/* else */
-	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp,
-	    "Column ", Tcl_GetString(columnIDObj), " out of range",
-	    NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+	    "Column %s out of range", Tcl_GetString(columnIDObj)));
+	Tcl_SetErrorCode(interp, "TTK", "TREE", "COLUMN", NULL);
 	return NULL;
     }
 
@@ -587,8 +583,9 @@ static TreeItem *FindItem(
     Tcl_HashEntry *entryPtr =  Tcl_FindHashEntry(&tv->tree.items, itemName);
 
     if (!entryPtr) {
-	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp, "Item ", itemName, " not found", NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"Item %s not found", itemName));
+	Tcl_SetErrorCode(interp, "TTK", "TREE", "ITEM", NULL);
 	return 0;
     }
     return Tcl_GetHashValue(entryPtr);
@@ -612,11 +609,11 @@ static TreeItem **GetItemListFromObj(
 	return NULL;
     }
 
-    items = (TreeItem**)ckalloc((nElements + 1)*sizeof(TreeItem*));
+    items = ckalloc((nElements + 1)*sizeof(TreeItem*));
     for (i = 0; i < nElements; ++i) {
 	items[i] = FindItem(interp, tv, elements[i]);
 	if (!items[i]) {
-	    ckfree((ClientData)items);
+	    ckfree(items);
 	    return NULL;
 	}
     }
@@ -658,7 +655,7 @@ static void TreeviewFreeColumns(Treeview *tv)
     if (tv->tree.columns) {
 	for (i = 0; i < tv->tree.nColumns; ++i)
 	    FreeColumn(tv->tree.columns + i);
-	ckfree((ClientData)tv->tree.columns);
+	ckfree(tv->tree.columns);
 	tv->tree.columns = 0;
     }
 }
@@ -687,8 +684,7 @@ static int TreeviewInitColumns(Tcl_Interp *interp, Treeview *tv)
      * Initialize columns array and columnNames hash table:
      */
     tv->tree.nColumns = ncols;
-    tv->tree.columns =
-	(TreeColumn*)ckalloc(tv->tree.nColumns * sizeof(TreeColumn));
+    tv->tree.columns = ckalloc(tv->tree.nColumns * sizeof(TreeColumn));
 
     for (i = 0; i < ncols; ++i) {
 	int isNew;
@@ -733,16 +729,16 @@ static int TreeviewInitDisplayColumns(Tcl_Interp *interp, Treeview *tv)
 
     if (!strcmp(Tcl_GetString(tv->tree.displayColumnsObj), "#all")) {
 	ndcols = tv->tree.nColumns;
-	displayColumns = (TreeColumn**)ckalloc((ndcols+1)*sizeof(TreeColumn*));
+	displayColumns = ckalloc((ndcols+1) * sizeof(TreeColumn*));
 	for (index = 0; index < ndcols; ++index) {
 	    displayColumns[index+1] = tv->tree.columns + index;
 	}
     } else {
-	displayColumns = (TreeColumn**)ckalloc((ndcols+1)*sizeof(TreeColumn*));
+	displayColumns = ckalloc((ndcols+1) * sizeof(TreeColumn*));
 	for (index = 0; index < ndcols; ++index) {
 	    displayColumns[index+1] = GetColumn(interp, tv, dcolumns[index]);
 	    if (!displayColumns[index+1]) {
-		ckfree((ClientData)displayColumns);
+		ckfree(displayColumns);
 		return TCL_ERROR;
 	    }
 	}
@@ -750,7 +746,7 @@ static int TreeviewInitDisplayColumns(Tcl_Interp *interp, Treeview *tv)
     displayColumns[0] = &tv->tree.column0;
 
     if (tv->tree.displayColumns)
-	ckfree((ClientData)tv->tree.displayColumns);
+	ckfree(tv->tree.displayColumns);
     tv->tree.displayColumns = displayColumns;
     tv->tree.nDisplayColumns = ndcols + 1;
 
@@ -1223,8 +1219,9 @@ static int ConfigureColumn(
     }
 
     if (mask & READONLY_OPTION) {
-	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp, "Attempt to change read-only option", NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"Attempt to change read-only option", -1));
+	Tcl_SetErrorCode(interp, "TTK", "TREE", "READONLY", NULL);
 	goto error;
     }
 
@@ -1828,7 +1825,7 @@ static int DrawSubtree(
 static int DrawForest(
     Treeview *tv, TreeItem *item, Drawable d, int depth, int row)
 {
-    while (item && row <= tv->tree.yscroll.last) {
+    while (item && row < tv->tree.yscroll.last) {
         row = DrawSubtree(tv, item, d, depth, row);
 	item = item->next;
     }
@@ -1913,11 +1910,10 @@ static int AncestryCheck(
     TreeItem *p = parent;
     while (p) {
 	if (p == item) {
-	    Tcl_ResetResult(interp);
-	    Tcl_AppendResult(interp,
-		    "Cannot insert ", ItemName(tv, item),
-		    " as a descendant of ", ItemName(tv, parent),
-		    NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "Cannot insert %s as descendant of %s",
+		    ItemName(tv, item), ItemName(tv, parent)));
+	    Tcl_SetErrorCode(interp, "TTK", "TREE", "ANCESTRY", NULL);
 	    return 0;
 	}
 	p = p->parent;
@@ -1986,7 +1982,7 @@ static int TreeviewChildrenCommand(
 	 */
 	for (i=0; newChildren[i]; ++i) {
 	    if (!AncestryCheck(interp, tv, newChildren[i], item)) {
-		ckfree((ClientData)newChildren);
+		ckfree(newChildren);
 		return TCL_ERROR;
 	    }
 	}
@@ -2022,7 +2018,7 @@ static int TreeviewChildrenCommand(
 	    child = newChildren[i];
 	}
 
-	ckfree((ClientData)newChildren);
+	ckfree(newChildren);
 	TtkRedisplayWidget(&tv->core);
     }
 
@@ -2291,8 +2287,8 @@ static int TreeviewIdentifyCommand(
 	return TCL_ERROR;
     }
 
-    if (   Tcl_GetIndexFromObj(interp, objv[2],
-		submethodStrings, "command", TCL_EXACT, &submethod) != TCL_OK
+    if (Tcl_GetIndexFromObjStruct(interp, objv[2], submethodStrings,
+		sizeof(char *), "command", TCL_EXACT, &submethod) != TCL_OK
         || Tcl_GetIntFromObj(interp, objv[3], &x) != TCL_OK
 	|| Tcl_GetIntFromObj(interp, objv[4], &y) != TCL_OK
     ) {
@@ -2319,9 +2315,7 @@ static int TreeviewIdentifyCommand(
 
 	case I_COLUMN :
 	    if (colno >= 0) {
-		char dcolbuf[16];
-		sprintf(dcolbuf, "#%d", colno);
-		Tcl_SetObjResult(interp, Tcl_NewStringObj(dcolbuf, -1));
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf("#%d", colno));
 	    }
 	    break;
 
@@ -2489,9 +2483,9 @@ static int TreeviewSetCommand(
 	for (columnNumber=0; columnNumber<tv->tree.nColumns; ++columnNumber) {
 	    Tcl_ListObjIndex(interp, item->valuesObj, columnNumber, &value);
 	    if (value) {
-		Tcl_ListObjAppendElement(interp, result,
+		Tcl_ListObjAppendElement(NULL, result,
 			tv->tree.columns[columnNumber].idObj);
-		Tcl_ListObjAppendElement(interp, result, value);
+		Tcl_ListObjAppendElement(NULL, result, value);
 	    }
 	}
 	Tcl_SetObjResult(interp, result);
@@ -2505,7 +2499,9 @@ static int TreeviewSetCommand(
 
     if (column == &tv->tree.column0) {
 	/* @@@ Maybe set -text here instead? */
-	Tcl_AppendResult(interp, "Display column #0 cannot be set", NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"Display column #0 cannot be set", -1));
+	Tcl_SetErrorCode(interp, "TTK", "TREE", "COLUMN_0", NULL);
 	return TCL_ERROR;
     }
 
@@ -2588,9 +2584,12 @@ static int TreeviewInsertCommand(
     objc -= 4; objv += 4;
     if (objc >= 2 && !strcmp("-id", Tcl_GetString(objv[0]))) {
 	const char *itemName = Tcl_GetString(objv[1]);
+
 	entryPtr = Tcl_CreateHashEntry(&tv->tree.items, itemName, &isNew);
 	if (!isNew) {
-	    Tcl_AppendResult(interp, "Item ",itemName," already exists",NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"Item %s already exists", itemName));
+	    Tcl_SetErrorCode(interp, "TTK", "TREE", "ITEM_EXISTS", NULL);
 	    return TCL_ERROR;
 	}
 	objc -= 2; objv += 2;
@@ -2647,8 +2646,10 @@ static int TreeviewDetachCommand(
     /* Sanity-check */
     for (i = 0; items[i]; ++i) {
 	if (items[i] == tv->tree.root) {
-	    Tcl_AppendResult(interp, "Cannot detach root item", NULL);
-	    ckfree((ClientData)items);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"Cannot detach root item", -1));
+	    Tcl_SetErrorCode(interp, "TTK", "TREE", "ROOT", NULL);
+	    ckfree(items);
 	    return TCL_ERROR;
 	}
     }
@@ -2658,7 +2659,7 @@ static int TreeviewDetachCommand(
     }
 
     TtkRedisplayWidget(&tv->core);
-    ckfree((ClientData)items);
+    ckfree(items);
     return TCL_OK;
 }
 
@@ -2694,8 +2695,10 @@ static int TreeviewDeleteCommand(
      */
     for (i=0; items[i]; ++i) {
 	if (items[i] == tv->tree.root) {
-	    ckfree((ClientData)items);
-	    Tcl_AppendResult(interp, "Cannot delete root item", NULL);
+	    ckfree(items);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"Cannot delete root item", -1));
+	    Tcl_SetErrorCode(interp, "TTK", "TREE", "ROOT", NULL);
 	    return TCL_ERROR;
 	}
     }
@@ -2719,7 +2722,7 @@ static int TreeviewDeleteCommand(
 	delq = next;
     }
 
-    ckfree((ClientData)items);
+    ckfree(items);
     TtkRedisplayWidget(&tv->core);
     return TCL_OK;
 }
@@ -2886,10 +2889,9 @@ static int TreeviewDragCommand(
 	left = right;
     }
 
-    Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp,
-	"column ", Tcl_GetString(objv[2]), " is not displayed",
-	NULL);
+    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+	"column %s is not displayed", Tcl_GetString(objv[2])));
+    Tcl_SetErrorCode(interp, "TTK", "TREE", "COLUMN_INVISIBLE", NULL);
     return TCL_ERROR;
 }
 
@@ -2953,9 +2955,8 @@ static int TreeviewSelectionCommand(
 	return TCL_ERROR;
     }
 
-    if (Tcl_GetIndexFromObj(interp, objv[2], selopStrings,
-	    "selection operation", 0, &selop) != TCL_OK)
-    {
+    if (Tcl_GetIndexFromObjStruct(interp, objv[2], selopStrings,
+	    sizeof(char *), "selection operation", 0, &selop) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -2988,7 +2989,7 @@ static int TreeviewSelectionCommand(
 	    break;
     }
 
-    ckfree((ClientData)items);
+    ckfree(items);
     TtkSendVirtualEvent(tv->core.tkwin, "TreeviewSelect");
     TtkRedisplayWidget(&tv->core);
 
@@ -3041,10 +3042,10 @@ static int TreeviewTagBindCommand(
 	     */
 	    if (mask & (~TreeviewBindEventMask)) {
 		Tk_DeleteBinding(interp, bindingTable, tag, sequence);
-		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, "unsupported event ", sequence,
-		    "\nonly key, button, motion, and virtual events supported",
-		    NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "unsupported event %s\nonly key, button, motion, and"
+		    " virtual events supported", sequence));
+		Tcl_SetErrorCode(interp, "TTK", "TREE", "BIND_EVENTS", NULL);
 		return TCL_ERROR;
 	    }
 	}
