@@ -922,7 +922,7 @@ ConfigureRestrictProc(
 	TkMacOSXUpdateClipRgn(winPtr);
 
 	 /*
-	  * Finally, generate and process expose events to redraw the window.
+	  * Generate and process expose events to redraw the window.
 	  */
 
 	HIRect bounds = NSRectToCGRect([self bounds]);
@@ -932,6 +932,12 @@ ConfigureRestrictProc(
 	if ([NSApp macMinorVersion] > 13) {
 	    [NSApp setIsDrawing:NO];
 	}
+
+	/*
+	 * Finally, unlock the main autoreleasePool.
+	 */
+	
+	[NSApp _unlockAutoreleasePool];
     }
 }
 
